"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.cleanDependencies = cleanDependencies;
exports["default"] = _default;
exports.isValidPackageName = isValidPackageName;

function _typeof2() {
  var data = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

  _typeof2 = function _typeof2() {
    return data;
  };

  return data;
}

function _slicedToArray2() {
  var data = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

  _slicedToArray2 = function _slicedToArray2() {
    return data;
  };

  return data;
}

function _errors() {
  var data = require("../../errors.js");

  _errors = function _errors() {
    return data;
  };

  return data;
}

function _util() {
  var data = require("./util.js");

  _util = function _util() {
    return data;
  };

  return data;
}

function _typos() {
  var data = _interopRequireDefault(require("./typos.js"));

  _typos = function _typos() {
    return data;
  };

  return data;
}

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var isBuiltinModule = require('is-builtin-module');

var strings = ['name', 'version'];
var dependencyKeys = [// npm registry will include optionalDependencies in dependencies and we'll want to dedupe them from the
// other fields first
'optionalDependencies', // it's seemingly common to include a dependency in dependencies and devDependencies of the same name but
// different ranges, this can cause a lot of issues with our determinism and the behaviour of npm is
// currently unspecified.
'dependencies', 'devDependencies'];

function isValidName(name) {
  return !name.match(/[\/@\s\+%:]/) && encodeURIComponent(name) === name;
}

function isValidScopedName(name) {
  if (name[0] !== '@') {
    return false;
  }

  var parts = name.slice(1).split('/');
  return parts.length === 2 && isValidName(parts[0]) && isValidName(parts[1]);
}

function isValidPackageName(name) {
  return isValidName(name) || isValidScopedName(name);
}

function _default(info, isRoot, reporter, warn) {
  if (isRoot) {
    for (var key in _typos()["default"]) {
      if (key in info) {
        warn(reporter.lang('manifestPotentialTypo', key, _typos()["default"][key]));
      }
    }
  } // validate name


  var name = info.name;

  if (typeof name === 'string') {
    if (isRoot && isBuiltinModule(name)) {
      warn(reporter.lang('manifestBuiltinModule', name));
    } // cannot start with a dot


    if (name[0] === '.') {
      throw new (_errors().MessageError)(reporter.lang('manifestNameDot'));
    } // cannot contain the following characters


    if (!isValidPackageName(name)) {
      throw new (_errors().MessageError)(reporter.lang('manifestNameIllegalChars'));
    } // cannot equal node_modules or favicon.ico


    var lower = name.toLowerCase();

    if (lower === 'node_modules' || lower === 'favicon.ico') {
      throw new (_errors().MessageError)(reporter.lang('manifestNameBlacklisted'));
    }
  } // validate license


  if (isRoot && !info["private"]) {
    if (typeof info.license === 'string') {
      var license = info.license.replace(/\*$/g, '');

      if (!(0, _util().isValidLicense)(license)) {
        warn(reporter.lang('manifestLicenseInvalid'));
      }
    } else {
      warn(reporter.lang('manifestLicenseNone'));
    }
  } // validate strings


  var _iterator = _createForOfIteratorHelper(strings),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var key = _step.value;
      var val = info[key];

      if (val && typeof val !== 'string') {
        throw new (_errors().MessageError)(reporter.lang('manifestStringExpected', key));
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  cleanDependencies(info, isRoot, reporter, warn);
}

function cleanDependencies(info, isRoot, reporter, warn) {
  // get dependency objects
  var depTypes = [];

  var _iterator2 = _createForOfIteratorHelper(dependencyKeys),
      _step2;

  try {
    for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
      var _type2 = _step2.value;
      var _deps2 = info[_type2];

      if (!_deps2 || (0, _typeof2()["default"])(_deps2) !== 'object') {
        continue;
      }

      depTypes.push([_type2, _deps2]);
    } // aggregate all non-trivial deps (not '' or '*')

  } catch (err) {
    _iterator2.e(err);
  } finally {
    _iterator2.f();
  }

  var nonTrivialDeps = new Map();

  for (var _i = 0, _depTypes = depTypes; _i < _depTypes.length; _i++) {
    var _depTypes$_i = (0, _slicedToArray2()["default"])(_depTypes[_i], 2),
        type = _depTypes$_i[0],
        deps = _depTypes$_i[1];

    for (var _i2 = 0, _Object$keys = Object.keys(deps); _i2 < _Object$keys.length; _i2++) {
      var name = _Object$keys[_i2];
      var version = deps[name];

      if (!nonTrivialDeps.has(name) && version && version !== '*') {
        nonTrivialDeps.set(name, {
          type: type,
          version: version
        });
      }
    }
  } // overwrite first dep of package with non-trivial version, remove the rest


  var setDeps = new Set();

  for (var _i3 = 0, _depTypes2 = depTypes; _i3 < _depTypes2.length; _i3++) {
    var _depTypes2$_i = (0, _slicedToArray2()["default"])(_depTypes2[_i3], 2),
        _type = _depTypes2$_i[0],
        _deps = _depTypes2$_i[1];

    for (var _i4 = 0, _Object$keys2 = Object.keys(_deps); _i4 < _Object$keys2.length; _i4++) {
      var _name = _Object$keys2[_i4];
      var _version = _deps[_name];
      var dep = nonTrivialDeps.get(_name);

      if (dep) {
        if (_version && _version !== '*' && _version !== dep.version && isRoot) {
          // only throw a warning when at the root
          warn(reporter.lang('manifestDependencyCollision', dep.type, _name, dep.version, _type, _version));
        }

        _version = dep.version;
      }

      if (setDeps.has(_name)) {
        delete _deps[_name];
      } else {
        _deps[_name] = _version;
        setDeps.add(_name);
      }
    }
  }
}
//# sourceMappingURL=validate.js.map
